package com.qmth.pdf.tools.util;

import org.springframework.http.*;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import java.nio.charset.StandardCharsets;
import java.util.Collections;
import java.util.Map;
import java.util.Objects;

/**
 * http请求工具类
 */
public class HttpClientUtil {

    private final RestTemplate restTemplate;

    public HttpClientUtil(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;
    }

    /**
     * 通用请求方法
     *
     * @param url     请求地址
     * @param method  请求方法
     * @param headers 请求头
     * @param params  请求参数（GET/DELETE时使用）
     * @param body    请求体（POST/PUT时使用）
     * @return 响应内容
     */
    public String request(String url, HttpMethod method,
            HttpHeaders headers,
            MultiValueMap<String, String> params,
            Object body) {

        // 设置 Accept-Charset 为 UTF-8
        headers.setAcceptCharset(Collections.singletonList(StandardCharsets.UTF_8));
        // 构建请求实体
        HttpEntity<?> requestEntity = new HttpEntity<>(body, headers);

        // 处理URL参数
        UriComponentsBuilder builder = UriComponentsBuilder.fromHttpUrl(url);
        if (params != null) {
            builder.queryParams(params);
        }

        // 发送请求
        ResponseEntity<byte[]> response = restTemplate.exchange(
                builder.build().toUri(),
                method,
                requestEntity,
                byte[].class
        );


        // 强制设置字符编码为 UTF-8
        return new String(Objects.requireNonNull(response.getBody()), StandardCharsets.UTF_8);
    }

    // GET请求
    public String get(String url, Map<String, String> headers, MultiValueMap<String, String> params) {
        HttpHeaders httpHeaders = new HttpHeaders();
        if (headers != null) {
            httpHeaders.setAll(headers);
        }
        return request(url, HttpMethod.GET, httpHeaders, params, null);
    }

    // POST请求
    public String postJson(String url, Map<String, String> headers, Object body) {
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setAll(headers);
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);
        return request(url, HttpMethod.POST, httpHeaders, null, body);
    }

    // POST请求（表单提交）
    public String postForm(String url, Map<String, String> headers, MultiValueMap<String, String> formData) {
        HttpHeaders httpHeaders = new HttpHeaders();
        if (headers != null) {
            httpHeaders.setAll(headers);
        }
        httpHeaders.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        return request(url, HttpMethod.POST, httpHeaders, null, formData);
    }
}